"""
Generic parsers for vmware configuration text files
"""

import numbers

from stellator.constants import *

class FileParserError(Exception):
    pass


class IndexedConfigEntry(object):
    """Indexed object

    Sortable parent class for indexed objects.
    """

    def __init__(self, index):
        self.index = int(index)

    def __cmp__(self, other):
        if isinstance(other, numbers.Number):
            return cmp(self.index, other)

        if isinstance(other, basestring):
            return cmp(self.index, int(other))

        return cmp(self.index, other.index)


class VMWareConfigFileParser(object):
    """Configuration parser

    Parser for vmware text configuration files, like .vmx or library inventory
    """
    def __init__(self, path):
        self.path = path
        self.meta = {}

    def load(self):
        try:
            with open(self.path, 'r') as fd:
                for line in sorted(line.strip() for line in fd.readlines()):
                    self.parse_line(line)
        except OSError, (ecode, emsg):
            raise ConfigFileError('Error reading {0}: (1}'.format(self.path, emsg))
        except IOError, (ecode, emsg):
            raise ConfigFileError('Error reading {0}: (1}'.format(self.path, emsg))

    def parse_line(self, line):
        """Split key, value line

        Expects lines to be of key = value format, calling self.parse_value for
        the detected entries. Ignores anything else.
        """
        try:
            key, value = [x.strip() for x in line.split('=', 1)]
        except ValueError:
            return
        value = value.strip('"')
        return self.parse_value(key, value)

    def parse_value(self, key, value):
        """Parse key, value pair

        Parses common options for all files. Returns True if value was parsed.
        """
        if key in CONFIG_META_KEYS:
            self.meta[CONFIG_META_KEYS[key]] = value
            return True

        return False



"""
Virtual machine configuration
"""

import glob
import os

from stellator.constants import *
from stellator.fileparser import VMWareConfigFileParser, IndexedConfigEntry, FileParserError


class VirtualMachineConfigurationSection(IndexedConfigEntry):
    """Common dot separated config section

    Parse keys in configuration and set to attributes of the object.
    """
    integer_keys = ()
    boolean_keys = ()
    key_map = {}

    def __init__(self, virtualmachine, index):
        super(VirtualMachineConfigurationSection, self).__init__(index)
        self.virtualmachine = virtualmachine

    def set(self, parts, value):
        if len(parts) > 1:
            return parts, value

        key = parts[0]

        if key in self.integer_keys:
            value = int(value)

        if key in self.boolean_keys:
            value = value == 'TRUE'

        if key in self.key_map:
            key = self.key_map[key]

        setattr(self, key, value)
        return key, value


class Interface(VirtualMachineConfigurationSection):
    """Ethernet network interface

    Virtualmachine Ethernet network interface
    """
    integer_keys = ( 'pciSlotNumber', 'generatedAddressOffset', )
    boolean_keys = ( 'present', 'startConnected', 'wakeOnPcktRcv', )
    key_map =  {
        'virtualDev': 'driver',
        'connectionType': 'connection_type',
        'startConnected': 'start_connected',
        'generatedAddress': 'mac_address',
        'wakeOnPcktRcv': 'wake_on_packet_receive',
        'addressType': 'address_type',
        'generatedAddressOffset': 'generated_address_offset',
        'pciSlotNumber': 'pci_slot_number',
    }

class PCIBridge(VirtualMachineConfigurationSection):
    """PCI bridge

    Virtualmachine PCI bridge
    """
    integer_keys = ( 'pciSlotNumber', 'functions', )
    boolean_keys = ( 'present', )
    key_map =  { 'virtualDev': 'virtual_device', 'pciSlotNumber': 'pci_slot_number', }


class USBPort(VirtualMachineConfigurationSection):
    """USB port

    Virtualmachine USB port attached
    """
    integer_keys = ( 'parent', 'speed', 'port', )
    boolean_keys = ( 'present', )
    key_map = { 'deviceType': 'device_type', }


class USB(VirtualMachineConfigurationSection):
    """USB support

    Section for USB support in VMs. Contains USBPort objects as well.
    """
    integer_keys = ( 'pciSlotNumber', )
    boolean_keys = ( 'present', )
    key_map = { 'pciSlotNumber': 'pci_slot_number', }

    def __init__(self, virtualmachine, index=0):
        super(USB, self).__init__(virtualmachine, index)
        self.interfaces = []


class VirtualMachine(VMWareConfigFileParser):
    """Vmware .vmx parser

    Parse .vmx file and give some functionality to control the VM.
    """
    def __init__(self, inventory, path):
        super(VirtualMachine, self).__init__(path)

        self.inventory = inventory

        # Set defaults for keys
        for key, value in VMX_KEY_MAP.items():
            if key in VMX_INTEGER_KEYS:
                setattr(self, value, 0)
            else:
                setattr(self, value, None)

        self.interfaces = []
        self.pci_bridges = []
        self.usb = USB(self)
        self.blockdevices = []

        self.load()

    def __repr__(self):
        return self.path

    def __cmp__(self, other):
        return cmp(self.path, other.path)

    @property
    def directory(self):
        """Return .vmx directory

        Return directory where .vmx file is
        """
        return os.path.dirname(self.path)

    @property
    def headless(self):
        """Is VM headless

        Headless VMs are not shown in GUI but can be run with vmrun and nogui
        """
        return self.inventory.find_vmx(self.path) is None

    @property
    def is_running(self):
        """Is VM running

        """
        try:
            running_vms = self.inventory.vmrun.running_vms()
        except VMRunError, emsg:
            # TODO - maybe we want something else here?
            raise VMRunError(emsg)

        return self.path in running_vms

    @property
    def autoresume_file(self):
        """Filename for autoresume

        Internal file to mark autoresume for VM.
        """
        return os.path.join(self.directory, self.inventory.config['autoresume_filename'])

    @property
    def autoresume(self):
        """Is VM autoresuming

        True if self.autoresume_file exists
        """
        return os.path.isfile(self.autoresume_file)

    @property
    def has_vmem(self):
        """Has .vmem file

        If VM is not running and has .vmem files, it is suspended.
        """
        return len(glob.glob('{0}/*.vmem'.format(self.directory))) > 0

    @property
    def status(self):
        """Return status string

        Returns status string for VM
        """
        if self.inventory.is_running(self):
            return 'running'

        elif self.has_vmem:
            return 'suspended'

        return 'stopped'

    def start(self):
        """Start VM

        """
        if self.is_running:
            return

        self.inventory.vmrun.start(self.path, headless=self.headless)

        if os.path.isfile(self.autoresume_file):
            try:
                os.unlink(self.autoresume_file)
            except OSError, (ecode, emsg):
                pass
            except OSError, (ecode, emsg):
                pass

    def suspend(self, autoresume=False):
        """Suspend VM

        If autoresume is True, a file is touched that allows scripts to
        resume the script automatically, for example after hibernation.

        This applies only to  headless VMs.
        """
        if not self.is_running:
            return

        self.inventory.vmrun.suspend(self.path)

        if autoresume and self.headless:
            try:
                open(self.autoresume_file, 'w').write('\n')
            except OSError, (ecode, emsg):
                pass
            except OSError, (ecode, emsg):
                pass

    def stop(self):
        """Stop VM

        Stop (shutdown) the VM
        """
        if not self.is_running:
            return

        self.inventory.vmrun.stop(self.path)

    def __get_interface__(self, index):
        for interface in self.interfaces:
            if interface == index:
                return interface

        interface = Interface(self, index)
        self.interfaces.append(interface)
        return interface

    def __get_pci_bridge__(self, index):
        for bridge in self.pci_bridges:
            if bridge == index:
                return bridge

        bridge = PCIBridge(self, index)
        self.pci_bridges.append(bridge)
        return bridge

    def __get_usb_interface__(self, index):
        for interface in self.usb.interfaces:
            if interface == index:
                return interface

        interface = USBPort(self, index)
        self.usb.interfaces.append(interface)
        return interface

    def parse_value(self, key, value):
        """Parse values

        Right now we skip most keys: implement rest if you are interested.
        """
        if super(VirtualMachine, self).parse_value(key, value):
            return

        if key in VMX_BOOLEAN_KEYS:
            value = value == 'TRUE'

        if key in VMX_INTEGER_KEYS:
            value = int(value)

        if key in VMX_KEY_MAP:
            key = VMX_KEY_MAP[key]
            setattr(self, key, value)

        else:
            parts = key.split('.')

            if parts[0] == 'usb':
                self.usb.set(parts[1:], value)

            elif parts[0][:8] == 'ethernet':
                index = int(parts[0][8:])
                parts = parts[1:]
                interface = self.__get_interface__(index)
                interface.set(parts, value)

            elif parts[0][:9] == 'pciBridge':
                index = int(parts[0][9:])
                parts = parts[1:]
                bridge = self.__get_pci_bridge__(index)
                bridge.set(parts, value)

            elif parts[0][:4] == 'usb:':
                try:
                    index = int(parts[0][4:])
                    parts = parts[1:]
                    usb = self.__get_usb_interface__(index)
                    usb.set(parts, value)
                except ValueError:
                    print parts

            else:
                pass
                # TODO - parse rest of options: these are not really interesting
                #print '{0}\n  {1} {2}'.format(key, type(value), value)



"""
Stellator configuration file parsing
"""

import configobj
import os

CONFIG_DIRECTORY = os.path.expanduser('~/Library/Application Support/Stellator')
DEFAULT_CONFIG_PATH = os.path.join(CONFIG_DIRECTORY, 'stellator.conf')


# Paths to vmware fusion install locations
VMWARE_FUSION_PATHS = (
    os.path.expanduser('~/Applications/VMware Fusion.app'),
    '/Applications/VMware Fusion.app',
)

def detect_vmware_fusion():
    for path in VMWARE_FUSION_PATHS:
        if os.path.isdir(path):
            return path
    raise StellatorConfigError('Could not detect installed vmware fusion')


DEFAULT_CONFIG = {
    'inventory': os.path.expanduser('~/Library/Application Support/VMware Fusion/vmInventory'),
    'application_path': detect_vmware_fusion(),
    'autoresume_filename': 'autoresume.stellator',
    'virtualmachines_path': os.path.expanduser('~/Documents/Virtual Machines.localized'),
}


class StellatorConfigError(Exception):
    pass


class StellatorConfig(dict):
    """User configuration

    Configuration file is in ~/Library/Application Support/Stellator/stellator.conf.
    Usually not needed but if you so wish, you can override things there. The default
    settings from DEFAULT_CONFIG should work.

    Note: following environment variables always overrides settings in config:

        VMWARE_INVENTORY: inventory
        VMWARE_DIRECTORY: virtualmachines_path

    """
    def __init__(self, path=DEFAULT_CONFIG_PATH):
        self.path = path

        self.update(DEFAULT_CONFIG)
        self.load()

        # Read environment and override settings if needed
        vm_inventory = os.environ.get('VMWARE_INVENTORY', None)
        if vm_inventory:
            self['inventory'] = vm_inventory

        vm_directory = os.environ.get('VMWARE_DIRECTORY', None)
        if vm_directory:
            self['virtualmachines_path'] = vm_directory

    def load(self):
        """Load user configuration

        """
        if not os.path.isdir(CONFIG_DIRECTORY):
            try:
                os.makedirs(CONFIG_DIRECTORY)
            except OSError, (ecode, emsg):
                raise StellatorConfigError('Error creating directory {0}: {1}'.format(CONFIG_DIRECTORY, emsg))
            except IOError, (ecode, emsg):
                raise StellatorConfigError('Error creating directory {0}: {1}'.format(CONFIG_DIRECTORY, emsg))

        config = configobj.ConfigObj(self.path)
        for key in DEFAULT_CONFIG.keys():
            if key in config:
                self[key] = config[key]





"""
Parser for user's vmware fusion virtual machine inventory
"""

import fnmatch
import os

from stellator.constants import *
from stellator.fileparser import VMWareConfigFileParser, IndexedConfigEntry, FileParserError
from stellator.virtualmachine import VirtualMachine
from stellator.vmrun import VMRunWrapper, VMRunError


class InventoryIndexField(IndexedConfigEntry):
    """Field for index

    Setting field for index
    """
    def __init__(self, inventory_index, index):
        super(InventoryIndexField, self).__init__(index)
        self.inventory_index = inventory_index

        self.default = False
        self.name = None
        self.value = None

    def __repr__(self):
        return '{0} {1} {2}'.format(self.inventory_index, self.name, self.value)

    def set(self, key, value):
        if key == 'default':
            self.default = value == 'True'
        else:
            setattr(self, key, value)


class InventoryIndex(IndexedConfigEntry):
    """Sortable inventory index

    Indexes the VMs shown in GUI
    """
    def __init__(self, inventory, index):
        super(InventoryIndex, self).__init__(index)

        self.inventory = inventory
        self.host_id = None
        self.vmx_path = None

        self.field_count = 0
        self.fields = []

    def __repr__(self):
        return '{0} {1} {2}'.format(self.index, self.host_id, self.vmx_path)

    def __get_field__(self, index):
        for instance in self.fields:
            if instance == index:
                return instance

        instance = InventoryIndexField(self, index)
        self.fields.append(instance)
        return instance

    @property
    def vmx_config(self):
        """Return VM config matching index

        """
        return self.inventory.find_vmx(self.vmx_path)

    def set(self, parts, value):
        if  parts == ['id']:
            self.vmx_path = value

        elif  parts == ['hostID']:
            self.host_id = value

        elif parts == [ 'field', 'count' ]:
            self.field_count = int(value)

        elif parts[0][:5] == 'field':
            field_index = int(parts[0][5:])
            key = '.'.join(parts[1:])
            field = self.__get_field__(field_index)
            field.set(key, value)

        else:
            print self, parts, value


class InventoryVirtualMachine(IndexedConfigEntry):
    """VM config in inventory

    Details for VM in inventory file. Virtualmachine object is in
    self.virtualmachine property.
    """
    def __init__(self, inventory, index):
        super(InventoryVirtualMachine, self).__init__(index)

        self.inventory = inventory
        self.vmx_path = None

        self.config = {
            'name': 'unknown',
        }

    def __repr__(self):
        return '{0} {1}'.format(self.config['name'], self.vmx_path)

    @property
    def virtualmachine(self):
        return VirtualMachine(self.inventory, self.vmx_path)

    def set(self, parts, value):
        key = '.'.join(parts)

        if key in INVENTORY_VM_BOOLEAN_FLAG_KEYS:
            value = value == 'TRUE'

        if key in INVENTORY_VM_INTEGER_KEYS:
            value = int(value)

        if key == 'UUID':
            value = value.replace(' ', '')

        if key == 'config':
            if value != '':
                self.vmx_path = value

        elif key in INVENTORY_VM_KEY_MAP:
            self.config[INVENTORY_VM_KEY_MAP[key]] = value

        else:
            print self, key, value


class Inventory(VMWareConfigFileParser):
    """VM inventory

    Inventory used by the GUI
    """
    def __init__(self, config):
        super(Inventory, self).__init__(config['inventory'])
        self.config = config
        self.vmrun = VMRunWrapper(self.config)
        self.index_count = 0

        self.indexes = []
        self.vmx_configs = []

        self.running_vms = []
        self.__running_vms_loaded = False

        self.load()

    def __get_index__(self, index):
        for instance in self.indexes:
            if instance == index:
                return instance

        instance = InventoryIndex(self, index)
        self.indexes.append(instance)
        return instance

    def __get_virtualmachine__(self, index):
        for instance in self.vmx_configs:
            if instance == index:
                return instance

        instance = InventoryVirtualMachine(self, index)
        self.vmx_configs.append(instance)
        return instance

    def __cleanup__vms__(self):
        for vm in [vm for vm in self.vmx_configs]:
            if vm.vmx_path is None:
                self.vmx_configs.remove(vm)
        self.vmx_configs.sort()

    @property
    def virtualmachines(self):
        """Return virtualmachines

        Return VMs matching the inventory config
        """
        return [vm.virtualmachine for vm in self.vmx_configs]

    def update_running_vmx(self):
        """Update list of running VMs

        Update running VMs list
        """
        self.running_vms = self.vmrun.running_vms()
        self.__running_vms_loaded = True

    def is_running(self, virtualmachine):
        """Check if VM is running

        This uses cached .running_vms - to update, run update_running_vmx() first.
        """
        if not self.__running_vms_loaded:
            self.update_running_vmx()
        return virtualmachine.path in self.running_vms

    def load(self):
        """Load inventory

        """
        rv = super(Inventory, self).load()
        self.indexes.sort()
        self.__cleanup__vms__()

        return rv

    def parse_value(self, key, value):
        """Parse values

        Parses inventory file values
        """
        if super(Inventory, self).parse_value(key, value):
            return

        elif key == 'index.count':
            self.index_count = int(value)
            return

        # Parse key prefix and index
        parts = key.split('.')

        if parts[0][:5] == 'index':
            index = int(parts[0][5:])
            parts = parts[1:]
            index = self.__get_index__(index)
            index.set(parts, value)

        if parts[0][:6] == 'vmlist':
            index = int(parts[0][6:])
            parts = parts[1:]
            virtualmachine = self.__get_virtualmachine__(index)
            virtualmachine.set(parts, value)

    def find_vmx(self, vmx_path):
        """Find vmx from inventory

        Returns vmx_config that matches path or None
        """
        for config in self.vmx_configs:
            if config.vmx_path == vmx_path:
                return config
        return None


class VirtualMachineFinder(list):
    """Find vmware virtual machines

    Find virtual machines both in specified path or in fusion inventory.

    Path defaults to DEFAULT_VMWARE_DIRECTORY, which either is taken from
    VMWARE_DIRECTORY environment variable or defaults to user's default
    VM location ~/Documents/Virtual Machines.localized.
    """

    def __init__(self, config=None):
        if config is None:
            config = StellatorConfig()
        self.config = config

        self.inventory = Inventory(config)
        self.path = config['virtualmachines_path']
        self.load()

    def is_running(self, virtualmachine):
        """Check if VM is running

        """
        return self.inventory.is_running(virtualmachine)

    def match_vm_names(self, patterns, case_sensitive=False):
        """Match VM names to patterns

        Return virtual machines that match the specified glob patterns.

        The match is case insensitive.
        """

        matches = []

        for virtualmachine in self:

            for pattern in patterns:
                if case_sensitive:
                   if fnmatch.fnmatch(virtualmachine.name, pattern):
                        matches.append(virtualmachine)
                        break
                else:
                    if fnmatch.fnmatch(virtualmachine.name.lower(), pattern.lower()):
                        matches.append(virtualmachine)
                        break

        return matches

    def load(self):
        """Load virtual machines

        Load virtual machines from directory and inventory
        """

        for root, dirs, files in os.walk(self.path):
            for filename in files:
                if os.path.splitext(filename)[1] == '.vmx':
                    self.append(VirtualMachine(self.inventory, os.path.join(root, filename)))


        for vm in self.inventory.virtualmachines:
            if vm not in self:
                self.append(vm)

        return [vm for vm in self]



"""
Constants for parsing vmware text configuration files
"""

# Generic config meta keys
CONFIG_META_KEYS = {
    '.encoding': 'encoding',
}

# Parsers for inventory
INVENTORY_VM_INTEGER_KEYS = (
    'ItemID',
    'ParentID',
    'SeqID',
)
INVENTORY_VM_BOOLEAN_FLAG_KEYS = (
    'IsCfgPathNormalized',
    'IsClone',
    'IsFavorite',
)
INVENTORY_VM_KEY_MAP = {
    'CfgVersion': 'configuration_version',
    'IsCfgPathNormalized': 'configuration_path_is_normalized',
    'IsClone': 'cloned',
    'IsFavorite': 'favorite',
    'ItemID': 'id',
    'ParentID': 'parent',
    'SeqID': 'sequence_id',
    'DisplayName': 'name',
    'UUID': 'uuid',
    'State': 'state',
}

# Constants for .vmx file parsing

VMX_INTEGER_KEYS = (
    'numvcpus',
    'memsize',
    'monitor.phys_bits_used',
    'svga.graphicsMemoryKB',
    'toolsInstallManager.updateCounter',
)
VMX_BOOLEAN_KEYS = (
    'cleanShutdown',
    'hpet0.present',
    'mem.hotadd',
    'mks.enable3d',
    'powerType.powerOff',
    'powerType.powerOn',
    'powerType.reset',
    'powerType.suspend',
    'replay.supported',
    'svga.guestBackedPrimaryAware',
    'softPowerOff',
    'vcpu.hotadd',
    'vmotion.checkpointFBSize',
    'vmotion.checkpointSVGAPrimarySize',
)
VMX_KEY_MAP = {
    'checkpoint.vmState': 'checkpoint_vmstate',
    'displayname': 'name',
    'displayName': 'name',
    'hpet0.present': 'hpet0_present',
    'memsize': 'memory',
    'numvcpus': 'cores',
    'nvram':  'nvram',
    'guestos': 'guest_os',
    'guestOS': 'guest_os',
    'gui.exitOnCLIHLT': 'gui_exit_on_client_halt',
    'gui.fullScreenAtPowerOn': 'gui_fullscreen_at_power_on',
    'gui.lastPoweredViewMode': 'gui_last_powered_view_mode',
    'gui.viewModeAtPowerOn': 'gui_view_mode_at_power_on',
    'cleanShutdown': 'clean_shutdown',
    'mem.hotadd': 'hotswap_mem',
    'migrate.hostlog': 'migrate_hostlog_filename',
    'mks.enable3d': 'enable_3d',
    'monitor.phys_bits_used': 'monitor_phys_bits_used',
    'replay.filename': 'replay_filename',
    'replay.supported': 'replay_supported',
    'vcpu.hotadd': 'hotswap_vcpu',
    'softPowerOff': 'soft_power_off',
    'powerType.powerOff': 'powertype_power_off',
    'powerType.powerOn': 'powertype_power_on',
    'powerType.reset': 'powertype_reset',
    'powerType.suspend': 'powertype_suspend',
    'extendedConfigFile': 'extended_config_file',
    'annotation': 'annotation',
    'config.version': 'compatibility_level',
    'svga.graphicsMemoryKB': 'svga_graphics_memory_kb',
    'svga.guestBackedPrimaryAware': 'svga_guestbacked_primate_aware',
    'tools.syncTime': 'tools_synctime',
    'tools.upgrade.policy': 'tools_upgrade_policy',
    'toolsInstallManager.updateCounter': 'tools_upgrade_counter',
    'uuid.bios': 'uuid_bios',
    'uuid.location': 'uuid_location',
    'virtualHW.version': 'virtual_hw_version',
    'virtualHW.productCompatibility': 'virtual_hw_product_compatibility',
    'vmotion.checkpointFBSize': 'vmotion_checkpoint_fb_size',
    'vmotion.checkpointSVGAPrimarySize': 'vmotion_checkpoint_svga_primary_size',
}

VMX_DEVICE_BOOLEAN_KEYS = (
    'present',
    'autodetect',
    'startConnected',
)
VMX_DEVICE_INTEGER_KEYS = (
    'pciSlotNumber',
)
VMX_DEVICE_KEY_MAP = {
    'deviceType': 'device_type',
    'fileName': 'filename',
    'startConnected': 'start_connected',
    'virtualDev': 'virtual_device',
}




import glob
import sys
import os
from setuptools import setup, find_packages
from stellator import __version__

setup(
    name = 'stellator',
    keywords = 'vmware fusion control scripts',
    description = 'Scripts to manage vmware fusion headless virtual machines',
    author = 'Ilkka Tuohela',
    author_email = 'hile@iki.fi',
    url = 'https://github.com/hile/stellator/',
    version = __version__,
    license = 'PSF',
    packages = find_packages(),
    scripts = glob.glob('bin/*'),
    install_requires = (
        'systematic>=4.4.2',
    ),
)




"""
Wrap vmrun calls
"""

import os
from subprocess import Popen, PIPE

VMRUN_RELATIVE_PATH = 'Contents/Library/vmrun'

class VMRunError(Exception):
    pass


class VMRunWrapper(object):
    """Wrap vmrun

    Execute some vmrun commands. Just what we need.
    """

    def __init__(self, config):
        self.path = os.path.join(config['application_path'], VMRUN_RELATIVE_PATH)
        if not os.access(self.path, os.X_OK):
            raise VMRunError('Not executable: {0}'.format(self.path))

    def __run__(self, args):
        command = [self.path] + args
        p = Popen(command, stdin=PIPE, stdout=PIPE, stderr=PIPE)
        stdout, stderr = p.communicate()
        return p.returncode, stdout, stderr

    def running_vms(self):
        rv, stdout, stderr = self.__run__(['list'])
        if rv != 0:
            raise VMRunError('Error running vmrun list: {0}'.format(stderr))

        return stdout.splitlines()[1:]

    def start(self, vmx_path, headless=False):
        if headless:
            rv, stdout, stderr = self.__run__(['start', vmx_path, 'nogui'])
        else:
            rv, stdout, stderr = self.__run__(['start', vmx_path])

        if rv != 0:
            raise VMRunError('Error starting {0}: {1}'.format(vmx_path, stderr))

    def suspend(self, vmx_path):
        rv, stdout, stderr = self.__run__(['suspend', vmx_path])

        if rv != 0:
            raise VMRunError('Error suspending {0}: {1}'.format(vmx_path, stderr))

    def stop(self, vmx_path):
        rv, stdout, stderr = self.__run__(['stop', vmx_path])

        if rv != 0:
            raise VMRunError('Error stopping {0}: {1}'.format(vmx_path, stderr))

